//
//  HRDeviceList.h
//  controllers_api
//
//  Created by Vladimir Starikh on 21.11.2022.
//  Copyright © 2022 HeliconSoft. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class RemoteCore;

typedef NS_ENUM(NSInteger, HRDeviceType) {
	HRDeviceType_Unknown,
	HRDeviceType_LocalCamera,	// local (usb) camera
	HRDeviceType_Network,		// network camera
	HRDeviceType_Stepper,		// stepper motor
	HRDeviceType_Server			// not used
};

@protocol HRDeviceListProtocol <NSObject>

@optional
- (void)infoAdded:(unsigned int)itemId;
- (void)infoRemoved:(unsigned int)itemId;
- (void)infoChanged:(unsigned int)itemId;
- (void)clearList;
- (void)selectedStateChanged:(unsigned int)infoId andState:(Boolean)state;
- (void)connectedStateChanged:(unsigned int)infoId andState:(Boolean)state;
- (void)connectingStateChanged:(unsigned int)infoId andState:(Boolean)state;

@end

@interface HRDeviceList : NSObject

- (instancetype)initWithCore:(RemoteCore *)core;

@property (weak) id<HRDeviceListProtocol> delegate;

- (void)updateUI;
#pragma mark device info
- (NSString *)deviceName:(unsigned int)infoId;
- (NSString *)deviceActualName:(unsigned int)infoId;
- (Boolean)connected:(unsigned int)infoId;
- (Boolean)connecting:(unsigned int)infoId;
- (Boolean)selected:(unsigned int)infoId;
- (HRDeviceType)deviceType:(unsigned int)infoId;
#pragma mark work
- (void)toggleSelection:(unsigned int)infoId;

@end

NS_ASSUME_NONNULL_END
