//
//  NSView+RemoteCore.h
//  controllers_api
//
//  Created by Vladimir Starikh on 17.11.2022.
//  Copyright © 2022 HeliconSoft. All rights reserved.
//

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class HRMainWindow;
@class HRLiveView;
@class HRDeviceList;
@class HRCameraSettings;
@class HRAdvCamSettings;
@class HRVideoRecording;
@class HRContinuousShooting;

@class HRPreferences;

@class HRDialog;
@class HRButton;

typedef NS_ENUM(NSInteger, HRDialogButtonRole) {
	HRDialogButtonRole_Neutral,
	HRDialogButtonRole_Positive,
	HRDialogButtonRole_Negative
};

@protocol RemoteCoreDelegate <NSObject>

@required

- (void)runQueue;

@optional

- (void)showTimeoutMessage;
- (void)showMessage:(HRDialog *)dlg;

@end

@interface RemoteCore : NSObject

@property (weak) id<RemoteCoreDelegate> delegate;

@property (strong, readonly) HRMainWindow *mainWnd;
@property (strong, readonly) HRLiveView *liveView;
@property (strong, readonly) HRDeviceList *deviceList;
@property (strong, readonly) HRCameraSettings *cameraSettings;
@property (strong, readonly) HRAdvCamSettings *advCamSettings;
@property (strong, readonly) HRVideoRecording *videoRec;
@property (strong, readonly) HRContinuousShooting *cs;

@property (strong, readonly) HRPreferences *preferences;

- (void)setCommonPath:(NSString *)path;
- (void)initLogger:(nonnull NSString *)fileName;
- (void)setPathToStoreImages:(NSString *)path;

- (void)initCore;
- (void)deinitCore;
- (void)createMockCamera;

@end

@interface HRDialog : NSObject

@property (strong) NSString *text;
@property (strong) NSArray<HRButton *> *buttons;

@end

@interface HRButton : NSObject

@property (strong) NSString *text;
@property (assign) HRDialogButtonRole buttonRole;

- (void)clicked;

@end

NS_ASSUME_NONNULL_END
