//
//  CameraSettingsViewController.m
//  remote
//
//  Created by Vladimir Starikh on 25.11.2022.
//

#import "CameraSettingsViewController.h"
#import "AppDelegate.h"
#import "ItemData.h"

#import <controllers_api/RemoteCore.h>
#import <controllers_api/HRCameraSettings.h>

@interface CameraSettingsViewController () <HRCameraSettingsProtocol>
{
	unsigned int _curExposureMode;
	unsigned int _curAperture;
	unsigned int _curShSpeed;
	unsigned int _curIso;
	unsigned int _curEv;
	unsigned int _curEV;
	unsigned int _curImgQual;
}

@property (weak) IBOutlet UIButton *btnExpMode;
@property (weak) IBOutlet UIButton *btnAperture;
@property (weak) IBOutlet UIButton *btnShSpeed;
@property (weak) IBOutlet UIButton *btnIso;
@property (weak) IBOutlet UIButton *btnEV;
@property (weak) IBOutlet UIButton *btnImgQual;
@property (weak) IBOutlet UILabel *lblBattery;

@property (weak) HRCameraSettings *camSettings;

@property (strong) NSMutableArray<ItemData *> * dataExposureMode;
@property (strong) NSMutableArray<ItemData *> * dataAperture;
@property (strong) NSMutableArray<ItemData *> * dataShSpeed;
@property (strong) NSMutableArray<ItemData *> * dataIso;
@property (strong) NSMutableArray<ItemData *> * dataEv;
@property (strong) NSMutableArray<ItemData *> * dataImgQual;

@end

@implementation CameraSettingsViewController

- (void)viewDidLoad {
    [super viewDidLoad];

	_dataExposureMode = [NSMutableArray new];
	_curExposureMode = -1;
	_dataAperture = [NSMutableArray new];
	_curAperture = -1;
	_dataShSpeed = [NSMutableArray new];
	_curShSpeed = -1;
	_dataIso = [NSMutableArray new];
	_curIso = -1;
	_dataEv = [NSMutableArray new];
	_curEv = -1;
	_dataImgQual = [NSMutableArray new];
	_curImgQual = -1;

	AppDelegate *appDelegate = (AppDelegate *)[UIApplication sharedApplication].delegate;
	RemoteCore *core = appDelegate.core;
	_camSettings = core.cameraSettings;
	_camSettings.delegate = self;
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];

	[_camSettings updateUI];
}

#pragma mark -
#pragma mark implement of HRCameraSettings protocol

- (void)controlStateChanged
{
	HR_ControlState state;

	state = [_camSettings controlState:HRCamSet_Controls_ExposureMode];
	_btnExpMode.enabled = state.enabled;

	state = [_camSettings controlState:HRCamSet_Controls_Aperture];
	_btnAperture.enabled = state.enabled;

	state = [_camSettings controlState:HRCamSet_Controls_ShutterSpeed];
	_btnShSpeed.enabled = state.enabled;

	state = [_camSettings controlState:HRCamSet_Controls_ISO];
	_btnIso.enabled = state.enabled;

	state = [_camSettings controlState:HRCamSet_Controls_ExpComp];
	_btnEV.enabled = state.enabled;

	state = [_camSettings controlState:HRCamSet_Controls_ImgQual];
	_btnImgQual.enabled = state.enabled;
}

- (void)addCBItem:(HRCamSet_Controls)type withName:(NSString *)name itemId:(unsigned int)itemId
{
	NSMutableArray<ItemData *> *data = nil;

	switch (type)
	{
		case HRCamSet_Controls_ExposureMode:
			data = _dataExposureMode;
			break;
		case HRCamSet_Controls_Aperture:
			data = _dataAperture;
			break;
		case HRCamSet_Controls_ShutterSpeed:
			data = _dataShSpeed;
			break;
		case HRCamSet_Controls_ISO:
			data = _dataIso;
			break;
		case HRCamSet_Controls_ExpComp:
			data = _dataEv;
			break;
		case HRCamSet_Controls_ImgQual:
			data = _dataImgQual;
			break;
		default:
			assert(false);
			return;
	}
	if (data == nil)
		return;

	ItemData *p = [[ItemData alloc] init];
	p.name = name;
	p.itemId = itemId;
	[data addObject:p];
}

- (void)clearCB:(HRCamSet_Controls)type
{
	switch (type)
	{
		case HRCamSet_Controls_ExposureMode:
			[_dataExposureMode removeAllObjects];
			_curExposureMode = -1;
			break;
		case HRCamSet_Controls_Aperture:
			[_dataAperture removeAllObjects];
			_curAperture = -1;
			break;
		case HRCamSet_Controls_ShutterSpeed:
			[_dataShSpeed removeAllObjects];
			_curShSpeed = -1;
			break;
		case HRCamSet_Controls_ISO:
			[_dataIso removeAllObjects];
			_curIso = -1;
			break;
		case HRCamSet_Controls_ExpComp:
			[_dataEv removeAllObjects];
			_curEv = -1;
			break;
		case HRCamSet_Controls_ImgQual:
			[_dataImgQual removeAllObjects];
			_curImgQual = -1;
			break;
		default:
			assert(false);
			return;
	}
}

- (void)setCurrentCBItem:(HRCamSet_Controls)type andItemId:(unsigned int)itemId
{
	switch (type)
	{
		case HRCamSet_Controls_ExposureMode:
			_curExposureMode = itemId;
			break;
		case HRCamSet_Controls_Aperture:
			_curAperture = itemId;
			break;
		case HRCamSet_Controls_ShutterSpeed:
			_curShSpeed = itemId;
			break;
		case HRCamSet_Controls_ISO:
			_curIso = itemId;
			break;
		case HRCamSet_Controls_ExpComp:
			_curEv = itemId;
			break;
		case HRCamSet_Controls_ImgQual:
			_curImgQual = itemId;
			break;
		default:
			assert(false);
			return;
	}
}

- (void)finishedUpdateProperty:(HRCamSet_Controls)type
{
	NSMutableArray<ItemData *> *data = nil;
	unsigned int curItem = -1;
	__block __weak UIButton *btn = nil;
	__block __weak CameraSettingsViewController *weakSelf = self;

	switch (type)
	{
		case HRCamSet_Controls_ExposureMode:
			data = _dataExposureMode;
			curItem = _curExposureMode;
			btn = _btnExpMode;
			break;
		case HRCamSet_Controls_Aperture:
			data = _dataAperture;
			curItem = _curAperture;
			btn = _btnAperture;
			break;
		case HRCamSet_Controls_ShutterSpeed:
			data = _dataShSpeed;
			curItem = _curShSpeed;
			btn = _btnShSpeed;
			break;
		case HRCamSet_Controls_ISO:
			data = _dataIso;
			curItem = _curIso;
			btn = _btnIso;
			break;
		case HRCamSet_Controls_ExpComp:
			data = _dataEv;
			curItem = _curEv;
			btn = _btnEV;
			break;
		case HRCamSet_Controls_ImgQual:
			data = _dataImgQual;
			curItem = _curImgQual;
			btn = _btnImgQual;
			break;
		default:
			assert(false);
			return;
	}
	if (data == nil)
		return;

	NSMutableArray *arr = [NSMutableArray new];
	if (data.count)
	{
		for (ItemData *p in data)
		{
			NSString *name = p.name;
			NSString *identefier = [NSString stringWithFormat:@"%d", p.itemId];
			UIAction *action = [UIAction actionWithTitle:name image:nil identifier:identefier handler:^(__kindof UIAction * _Nonnull action) {
				HRCamSet_Controls hrType;
				if (btn == weakSelf.btnExpMode)
					hrType = HRCamSet_Controls_ExposureMode;
				else if (btn == weakSelf.btnAperture)
					hrType = HRCamSet_Controls_Aperture;
				else if (btn == weakSelf.btnShSpeed)
					hrType = HRCamSet_Controls_ShutterSpeed;
				else if (btn == weakSelf.btnIso)
					hrType = HRCamSet_Controls_ISO;
				else if (btn == weakSelf.btnEV)
					hrType = HRCamSet_Controls_ExpComp;
				else if (btn == weakSelf.btnImgQual)
					hrType = HRCamSet_Controls_ImgQual;
				else
				{
					assert(false);
					return;
				}
				[weakSelf.camSettings changeCameraProperty:hrType andId:(unsigned int)action.identifier.longLongValue];
			}];
			if (p.itemId == curItem)
				action.state = UIMenuElementStateOn;
			[arr addObject:action];
		}
	}
	else
	{
		NSString *name;
		switch (type)
		{
			case HRCamSet_Controls_ExposureMode:
				name = @"Exposure mode";
				break;
			case HRCamSet_Controls_Aperture:
				name = @"Aperture";
				break;
			case HRCamSet_Controls_ShutterSpeed:
				name = @"Shutter speed";
				break;
			case HRCamSet_Controls_ISO:
				name = @"ISO";
				break;
			case HRCamSet_Controls_ExpComp:
				name = @"Exposure compensation";
				break;
			case HRCamSet_Controls_ImgQual:
				name = @"Image quality";
				break;
			default:
				assert(false);
				break;
		}
		NSString *identefier = name;
		UIAction *action = [UIAction actionWithTitle:name image:nil identifier:identefier handler:^(__kindof UIAction * _Nonnull action) {
		}];
		action.attributes = UIMenuElementAttributesDisabled;
		[arr addObject:action];
	}
	UIMenu *mn = [UIMenu menuWithTitle:@"Camera property" children:arr];
	btn.menu = mn;
}

- (void)batteryValChanged:(int)val
{
	_lblBattery.text = [NSString stringWithFormat:@"Battery: %d", val];
}

@end
