//
//  ContinuousShootingViewController.m
//  remote
//
//  Created by Vladimir Starikh on 13.02.2023.
//

#import "ContinuousShootingViewController.h"
#import "AppDelegate.h"
#import "MenuPlaceholder.h"

#import <controllers_api/RemoteCore.h>
#import <controllers_api/HRContinuousShooting.h>

@interface ContinuousShootingViewController () <HRContinuousShootingProtocol, MenuPlaceholderDelegate>

@property (weak) IBOutlet UIButton *btnShootingMode;
@property (weak) IBOutlet UITextField *edNumShots;
@property (weak) IBOutlet UIButton *btnStart;
@property (weak) IBOutlet UIButton *btnStop;

@property (strong) MenuPlaceholder *dataSM;

@property (weak) HRContinuousShooting *cs;

@end

@implementation ContinuousShootingViewController

- (void)viewDidLoad
{
	[super viewDidLoad];

	AppDelegate *app = (AppDelegate *)[UIApplication sharedApplication].delegate;
	RemoteCore *core = app.core;

	_cs = core.cs;
	_cs.delegate = self;

	_dataSM = [[MenuPlaceholder alloc] initWithButton:_btnShootingMode];
	_dataSM.delegate = self;
	_dataSM.menuName = @"Shooting mode";
	_dataSM.itemPlaceholderName = @"Shooting mode";
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];

	[_cs updateUI];
	NSUInteger numShots = _cs.numShots;
	_edNumShots.text = [NSString stringWithFormat:@"%u", (unsigned int)numShots];
}

- (IBAction)start:(id)sender
{
	NSString *text = _edNumShots.text;
	_cs.numShots = (NSUInteger)text.integerValue;

	[_cs startShooting];
}

- (IBAction)stop:(id)sender
{
	[_cs stopShooting];
}

#pragma mark - implement of HRContinuousShootingProtocol

- (void)csControlStateChanged
{
	{
		HR_ControlState state = [_cs controlState:HRContShooting_Controls_ShootingMode];
		_btnShootingMode.enabled = state.enabled;
	}
	{
		HR_ControlState state = [_cs controlState:HRContShooting_Controls_NumShots];
		_edNumShots.enabled = state.enabled;
	}
	{
		HR_ControlState state = [_cs controlState:HRContShooting_Controls_Start];
		_btnStart.enabled = state.enabled;
	}
	{
		HR_ControlState state = [_cs controlState:HRContShooting_Controls_Stop];
		_btnStop.enabled = state.enabled;
	}
}

- (void)csAddCBItem:(HRContShooting_Controls)type withName:(NSString *)name itemId:(unsigned int)itemId
{
	MenuPlaceholder *data;
	switch (type)
	{
		case HRContShooting_Controls_ShootingMode:
			data = _dataSM;
			break;
		default:
			assert(false);
			return;
	}
	[data itemAdded:name withId:itemId];
}

- (void)csClearCB:(HRContShooting_Controls)type
{
	MenuPlaceholder *data;
	switch (type)
	{
		case HRContShooting_Controls_ShootingMode:
			data = _dataSM;
			break;
		default:
			assert(false);
			return;
	}
	[data clearMenu];
}

- (void)csSetCurrentCBItem:(HRContShooting_Controls)type andItemId:(unsigned int)itemId
{
	MenuPlaceholder *data;
	switch (type)
	{
		case HRContShooting_Controls_ShootingMode:
			data = _dataSM;
			break;
		default:
			assert(false);
			return;
	}
	[data setCurrentItem:itemId];
}

#pragma mark - implement of ManuPlaceHolderDelegate

- (void)menuPlaceholder:(MenuPlaceholder *)mp itemSelected:(unsigned int)itemId
{
	if (mp == _dataSM)
	{
		[_cs setShootingMode:itemId];
	}
	else
		assert(false);
}

@end
