//
//  DeviceListViewController.m
//  remote
//
//  Created by Vladimir Starikh on 22.11.2022.
//

#import "DeviceListViewController.h"
#import "AppDelegate.h"

#import <controllers_api/RemoteCore.h>
#import <controllers_api/HRDeviceList.h>

@interface DeviceListViewController () <UITableViewDelegate, UITableViewDataSource, HRDeviceListProtocol>

@property (weak) IBOutlet UIButton *btnBack;
@property (weak) IBOutlet UITableView *table;

@property (weak) HRDeviceList *devList;

@property (strong) NSMutableArray<NSNumber *> *devices;

@end

@implementation DeviceListViewController

- (void)viewDidLoad {
	[super viewDidLoad];

	AppDelegate *appDelegate = (AppDelegate *)[UIApplication sharedApplication].delegate;

	RemoteCore *core = appDelegate.core;
	_devList = core.deviceList;
	_devList.delegate = self;

	_devices = [NSMutableArray new];
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];

	[_devList updateUI];
}

- (IBAction)createMock:(id)sender
{
	AppDelegate *appDelegate = (AppDelegate *)[UIApplication sharedApplication].delegate;

	RemoteCore *core = appDelegate.core;
	[core createMockCamera];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
	return 1;
}

- (NSString*)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section
{
	return @"Devices";
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
	return [_devices count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
	UITableViewCell *cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:nil];
	unsigned int infoId = (unsigned int)indexPath.row;
	NSString *type;
	switch ([_devList deviceType:infoId])
	{
		case HRDeviceType_Unknown:
			type = @"Unknown";
			break;
		case HRDeviceType_LocalCamera:
			type = @"USB";
			break;
		case HRDeviceType_Network:
			type = @"Network";
			break;
		case HRDeviceType_Stepper:
			type = @"Stepper";
			break;
		case HRDeviceType_Server:
			type = @"Server";
			break;
	}
	NSString *selected = [_devList selected:infoId] ? @"Selected" : @"";
	NSString *name = [NSString stringWithFormat:@"%@(%@) %@", [_devList deviceActualName:infoId], type, selected];
	cell.textLabel.text = name;
	cell.tag = _devices[infoId].unsignedIntValue;
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
	unsigned int infoId = (unsigned int)indexPath.row;
	[_devList toggleSelection:infoId];
}

#pragma mark -
#pragma mark implement of HRDeviceListProtocol

- (void)infoAdded:(unsigned int)itemId
{
	NSNumber *val = [NSNumber numberWithUnsignedInt:itemId];
	[_devices addObject:val];

	[_table reloadData];
}

- (void)infoRemoved:(unsigned int)itemId
{
	NSNumber *val = [NSNumber numberWithUnsignedInteger:itemId];
	[_devices removeObject:val];

	[_table reloadData];
}

- (void)infoChanged:(unsigned int)itemId
{
	[_table reloadData];
}

- (void)clearList
{
	[_devices removeAllObjects];

	[_table reloadData];
}

- (void)selectedStateChanged:(unsigned int)infoId andState:(Boolean)state
{
	[_table reloadData];
}

- (void)connectedStateChanged:(unsigned int)infoId andState:(Boolean)state
{
	[_table reloadData];
}

- (void)connectingStateChanged:(unsigned int)infoId andState:(Boolean)state
{
	[_table reloadData];
}

@end
