//
//  PrefGeneralViewController.m
//  remote
//
//  Created by Vladimir Starikh on 05.12.2022.
//

#import "PrefGeneralViewController.h"
#import "../AppDelegate.h"
#import "../MenuPlaceholder.h"

#import <controllers_api/RemoteCore.h>
#import <controllers_api/HRPreferences.h>
#import <controllers_api/HRPrefGeneral.h>

@interface PrefGeneralViewController () <HRPrefGeneralDelegate, MenuPlaceholderDelegate>
{
}

@property (weak) IBOutlet UIButton *btnActivateLVOnStartUp;

@property (weak) HRPrefGeneral *prefGeneral;

@property (strong) MenuPlaceholder *dataActivateLVOnStartUp;

@end

@implementation PrefGeneralViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view from its nib.

	AppDelegate *appDelegate = (AppDelegate *)[UIApplication sharedApplication].delegate;
	RemoteCore *core = appDelegate.core;
	HRPreferences *preferences = core.preferences;
	_prefGeneral = preferences.general;
	_prefGeneral.delegate = self;

	_dataActivateLVOnStartUp = [[MenuPlaceholder alloc] initWithButton:_btnActivateLVOnStartUp];
	_dataActivateLVOnStartUp.delegate = self;
	_dataActivateLVOnStartUp.menuName = @"Active LiveView on Startup";
	_dataActivateLVOnStartUp.itemPlaceholderName = @"Active LiveView on Startup";
}

- (void)viewWillAppear:(BOOL)animated
{
	[super viewWillAppear:animated];

	[_prefGeneral updateUI];
}

- (void)viewDidDisappear:(BOOL)animated
{
	[super viewDidDisappear:animated];

	[_prefGeneral accept];
}

#pragma mark - implement of HRPrefGeneralDelegate

- (void)prefGenAddCBItem:(HRPrefGeneral_Controls)type withName:(NSString *)name andId:(unsigned int)itemId
{
	MenuPlaceholder *data;

	switch (type)
	{
		case HRPrefGeneral_Controls_ActivateLiveViewOnStartup:
			data = _dataActivateLVOnStartUp;
			break;

		default:
			assert(false);
			return;
	}
	[data itemAdded:name withId:itemId];
}

- (void)prefGenClearCB:(HRPrefGeneral_Controls)type
{
	MenuPlaceholder *data;

	switch (type)
	{
		case HRPrefGeneral_Controls_ActivateLiveViewOnStartup:
			data = _dataActivateLVOnStartUp;
			break;

		default:
			assert(false);
			return;
	}
	[data clearMenu];
}

- (void)prefGenSetCurItemInCB:(HRPrefGeneral_Controls)type andId:(unsigned int)itemId
{
	MenuPlaceholder *data;

	switch (type)
	{
		case HRPrefGeneral_Controls_ActivateLiveViewOnStartup:
			data = _dataActivateLVOnStartUp;
			break;

		default:
			assert(false);
			return;
	}
	[data setCurrentItem:itemId];
}

#pragma mark - implement of MenuPlaceholderDelegate

- (void)menuPlaceholder:(MenuPlaceholder *)mp itemSelected:(unsigned int)itemId
{
	HRPrefGeneral_Controls hrType;

	if (mp == _dataActivateLVOnStartUp)
		hrType = HRPrefGeneral_Controls_ActivateLiveViewOnStartup;
	else
	{
		assert(false);
		return;
	}
	[_prefGeneral setCurrentItem:hrType andItemId:itemId];
}

@end
